# -*- coding: utf-8 -*-
import datetime
import time
from .school import School
import logging


class User:
    def __init__(self, api, email):
        self.api = api
        self._info = None
        self.email = email.strip()
        #        self._email = self.email = email.strip()
        self._name = None
        #        self._name = None
        self._id = None
        self._reportcard = None
        self._exists = None
        self._notified = None
        self._school = None
        self._courses = None
        self.logger = logging.getLogger(__name__)

    @property
    def reportcard(self):
        if not self._reportcard:
            if self.info:
                self._reportcard = self.api.get_user_report_card(self.id)
        return self._reportcard

    @property
    def name(self):
        # name getter property
        if not self._name and self.info:
            self._name = self.info.get('name').strip()
        return self._name

    @name.setter
    def name(self, name):
        # name setter property
        if not self.info:
            # we allow setting the name only if the user does not exist already
            # on the server side
            self._name = name
        else:
            self.logger.error("Can't set the name of {} as it already exists on Teachable".
                              format(self._name))

    @property
    def school(self):
        # school getter property
        if not self._school:
            self._school = School(self.api)
        return self._school

    @property
    def notified(self):
        # notified getter property
        if not self._notified:
            self._notified = self.api.get_last_notif(self.email)
        return self._notified

    @notified.setter
    def notified(self, newdate):
        # notified setter property
        self.api.set_last_notif(self.email, newdate)
        self._notified = newdate

    #    @property
    #    def email(self):
    #        # email getter property
    #        if not self._email and self.info:
    #            self._email = self.info.get('email').strip()
    #        return self._email
    #
    #    @email.setter
    #    def email(self, email):
    #        if self.api.check_email(email):
    #            print('Setting email')
    #            self._email = email
    #        else:
    #            self._email = None
    #        print(self._email)
    #        return self._email

    @property
    def id(self):
        if not self._id:
            if self.info:
                self._id = self.info.get('id')
        return self._id

    @property
    def exists(self):
        if not self._exists:
            if self.info:
                self._exists = True
            else:
                self._exists = False
        return self._exists

    @property
    def info(self):
        if not self._info:
            self._info = self.api.get_user_info(self.email)
            if not self._info:
                pass
                # self.logger.info('User with {} email doesn\'t exist in this school yet'.format(self.email))
        return self._info

    @property
    def courses(self):
        # courses getter property
        if not self._courses and self.info:
            self._courses = self.api.get_enrolled_courses(self.id)
        return self._courses
    
    def create(self, course=None):
        """Create the user on the server side, if the user doesn't exist and it
        has valid email """
        if self.exists is not True:
            if self.api.check_email(self.email):
                new = self.api.add_user_to_school(self, course)
                if new['message'] == 'Users imported':
                    # self.logger.info('Waiting Teachable to update backend')
                    time.sleep(1)
                    self.api.usecache = False
                    # self.logger.debug('Refreshing info for user {}'.format(self.email))
                    property().getter(self.info)
                    self.api.usecache = True
            else:
                new = {'message': '{} is not a valid email address'.format(self.email)}
        else:
            new = {'message': 'user with email {} already exists'.format(self.email)}
        return new

    def get_summary_stats(self, course_id=0):
        # school is not really needed here, since every user is only part of a specific school
        """Returns a list of lists with a summary stat for the specific user"""
        stats = []
        now = datetime.datetime.today()
        for (key, course_data) in self.reportcard.items():
            if key != 'meta':
                current_course_id = course_data.get('course_id')
                if ((course_id != 0) and current_course_id == course_id) or course_id == 0:
                    course = self.school.get_course_with_id(current_course_id)
                    percentage = course_data.get('percent_complete')
                    update_time = datetime.datetime.strptime(course_data.get('updated_at'), '%Y-%m-%dT%H:%M:%SZ')
                    days_since_last = (now - update_time).days
                    updated_at = update_time.strftime('%Y-%m-%d %H:%M:%S')
                    stats.append([self.name, self.email, course.name, updated_at,
                                  str(percentage), days_since_last])
        return stats

        # user_ordered_list = sorted(output, key=itemgetter('course_percentage'), reverse=True)

    # def generate_student_progress_list(self,course, output): get_course_curriculum(course_id)
    # current_lecture_title, current_section_title = get_latest_viewed_title(course, course_id) output.append({
    # 'course_id': course_id, 'course_name': course_list[course_data].get('name'), 'course_percentage': course.get(
    # 'percent_complete'), 'course_current_lecture': current_lecture_title, 'course_current_section':
    # current_section_title})

    # def get_latest_viewed_title(self, course_id):
    #     courseStats = self.getStatisticsForCourse(course_id)
    #     completedLectures = courseStats.get('completed_lecture_ids')
    #     latestLectureId = max(completedLectures)
    #     ordered_id_list = []
    #
    #     if course.get('completed_lecture_ids'):
    #         course_curriculum = curriculum.get(course_id)
    #
    #         for section in sections:
    #             for lecture in section.get('lectures'):
    #                 ordered_id_list.append(lecture.get('id'))
    #
    #         completed_lectures = course.get('completed_lecture_ids')
    #
    # # this function to order the completed_lectures looks, # but will fail if one of the lectures gets deleted (and
    # won't appear in ordered_id_list) #ordered_completed_lectures = sorted(completed_lectures,
    # key=ordered_id_list.index) ordered_completed_lectures = sorted(completed_lectures, key=lambda k: (
    # ordered_id_list.index(k) if k in ordered_id_list else -1))
    #
    #         for section in sections:
    #             lecture_id = find(section.get('lectures'), 'id', ordered_completed_lectures[-1])
    #             if lecture_id >= 0:
    #                 lecture_name = section.get('lectures')[lecture_id].get('name')
    #                 section_name = section.get('name')
    #                 return lecture_name, section_name
    #     return '', ''

    def get_detailed_stats(self):
        """Returns a list of lists with detailed stats for the specific user

        :return: a list of lessons
        """
        data = []
        stats = self.api.get_user_course_report(self.id)
        lectures_stats = stats.get('lecture_progresses')
        for lectureProgress in lectures_stats:
            completed_date = datetime.datetime.strptime(lectureProgress.get('completed_at'), '%Y-%m-%dT%H:%M:%SZ')
            course_id = lectureProgress.get('course_id')
            lecture_id = lectureProgress.get('lecture_id')
            course = self.school.get_course_with_id(course_id)
            lecture = course.get_lecture_with_id(lecture_id)
            str_cdate = completed_date.strftime("%Y-%m-%d %H:%M:%S")
            data.append([self.name, self.email, str_cdate, course.name,
                         lecture.name, lecture.duration_as_text])
        return data

    def is_enrolled_to_course(self, course_id):
        """Returns true if user is enrolled to courseId"""
        return self.api.check_enrollment_to_course(self.id, course_id)

    def __str__(self):
        return '{} <{}>'.format(self.name, self.email)

    def __repr__(self):
        return '<User({})>'.format(self.email)

    def enroll(self, course):
        return self.api.enroll_user_to_course(self, course)

    def unenroll(self, course):
        return self.api.unenroll_user_from_course(self, course)



import schedule
import time
import subprocess


def remind():
    subprocess.run(["teachable_remind"])
    return


def statements():
    subprocess.run(["teachable_statements", "-e"])
    return


def main():
    schedule.every().saturday.at("13:15").do(remind)
    schedule.every().saturday.at("13:15").do(statements)

    while True:
        schedule.run_pending()
        time.sleep(1)


if __name__ == "__main__":
    main()



# coding: utf8
import argparse
import logging

from ..api import Teachable
from ..school import School


def parse_arguments():
    parser = argparse.ArgumentParser(description='''Get a Leaderboard CSV in just
    one command. 
    
    It will save as many leaderboards CSV as you have courses.''')
    parser.add_argument('--search', '-s', nargs='?', default='',
                        help='''Searches specific text in the name of the course''')
    parser.add_argument('--output-file', '-o', nargs='?', default='',
                        dest='filename', help='''Specify the output file''')

    args = parser.parse_args()
    return args


def leaderboard_app(args):
    api = Teachable()
    logger = logging.getLogger(__name__)
    if args.search:
        courses = api.find_courses(args.search)
    else:
        courses = api.find_courses('')

    if courses:
        for course in courses:
            api.get_leaderboard_csv(course, args.filename)
    else:
        logger.info('Sorry no courses found with the term \'{search}\''.format(search=args.search))


def main():
    args = parse_arguments()
    leaderboard_app(args)


if __name__ == '__main__':
    main()



# coding: utf8
import argparse
import datetime
import logging
import os
import os.path
from email.utils import formataddr

import pytablewriter as ptw

from ..utils.email_utils import Email
from ..utils.email_utils import EmailConnection
from ..utils.email_utils import render_template
from ..api import Teachable
from ..user import User


def parse_arguments():
    parser = argparse.ArgumentParser(description='''Polls Teachable and sends
    reminders to those that haven't started a course or haven't done a lesson
    in a week.''')

    parser.add_argument('--dryrun', '-d', action='store_true',
                        default='False', help='''Don't send the messages for 
                        real, just do a dry run''')

    arguments = parser.parse_args()
    return arguments


def remind_app(args):
    """Main application"""
    api = Teachable()
    logger = logging.getLogger(__name__)
    config = api.config
    #   override_mail = 'stefano.mosconi@britemind.io'
    defaults = config['DEFAULT']
    site_url = defaults['site_url']
    smtp_pwd = defaults['smtp_pwd']
    smtp_user = defaults['smtp_user']
    smtp_port = defaults['smtp_port']
    smtp_server = defaults['smtp_server']
    smtp_from = defaults['smtp_from']
    templates_dir = api.TEACHABLE_TEMPLATES_DIR
    now = datetime.datetime.now()
    logger.info('Connecting to email server ({})'.format(smtp_server))
    server_str = smtp_server + ':' + str(smtp_port)
    server = EmailConnection(server_str, smtp_user, smtp_pwd)
    today = datetime.date.today()

    for section in config.keys():
        if section != 'DEFAULT':
            alert_days_int = int(config[section]['alert_days'])
            warning_days = int(config[section]['warning'])
            notif_freq = int(config[section]['freq'])
            max_inactivity = int(config[section]['max_inactivity'])
            try:
                exclude = config[section]['exclude'].replace(" ","").split(',')
            except KeyError:
                exclude = []
            data = []
            warn_students = []
            users_mails = [x.email for x in api.find_many_users(config[section]['emailsearch']) if x.email not in exclude]
            # First send a reminder to all that need it
            for user_mail in users_mails:
                user = User(api, user_mail)
                since_last_notif = (today - user.notified).days
                summary_stats = user.get_summary_stats(int(config[section]['course_id']))
                # Saves the overall stats separately
                data += summary_stats
                if summary_stats:
                    name, email_addr, course, updated_at, completed, days_since = summary_stats[0]
                    message = ''
                    updated_at = datetime.datetime.strptime(updated_at, "%Y-%m-%d %H:%M:%S")
                    completed = int(completed)
                    firstname = name.split()[0]
                    from_addr = formataddr((smtp_from, smtp_user))
                    to_addr = formataddr((name, email_addr))
                    # to_addr = formataddr((name, override_mail))
                    cc_addr = None
                    msg_dict = {'firstname': firstname, 'days_since': days_since, 'course': course,
                                'date': updated_at, 'url': site_url, 'name_from': smtp_from}
                    not_started = (completed == 0 and days_since > alert_days_int)
                    inactive = (days_since > alert_days_int and completed < 100)
                    flagged = (days_since > warning_days and completed < 100)
                    if not_started or flagged:
                        warn_students += summary_stats
                    if completed > 0 and inactive:
                        subject = 'Don\'t forget the {course} course'.format(course=course)
                        message = render_template(os.path.join(templates_dir, 'email_inactive.txt'),
                                                  msg_dict)
                    elif not_started:
                        subject = 'Don\'t forget the {course} course'.format(course=course)
                        message = render_template(os.path.join(templates_dir, 'email_notstarted.txt'),
                                                  msg_dict)
                    if message:
                        mail = Email(from_=from_addr, to=to_addr, cc=cc_addr,
                                     subject=subject, message=message, message_encoding="utf-8")
                        if since_last_notif >= notif_freq and days_since <= max_inactivity:
                            if args.dryrun is not True:
                                logger.info('Sending mail to {}'.format(to_addr))
                                server.send(mail, bcc=smtp_user)
                                user.notified = today
                            else:
                                logger.info('[DRYRUN] Not sending email to {}'.format(to_addr))
                    else:
                        logger.info('No reminder for ' + to_addr)
            if data:
                # Now send the overall stats to the specified contact person
                headers = ['User', 'Email', 'Course', 'Updated at',
                           'Completed (%)', 'Days since last lesson']
                writer = ptw.ExcelXlsxTableWriter()
                # Because honestly MS Gothic sucks as a font
                writer.format_table['cell']['font_name'] = 'Calibri'
                writer.format_table['cell']['font_size'] = 12
                writer.format_table['header']['font_name'] = 'Calibri'
                writer.format_table['header']['font_size'] = 12
                writer.table_name = 'Teachable Stats'
                writer.headers = headers
                writer.value_matrix = data
                ofile = section + '_' + str(now.strftime('%d-%m-%y')) + '.xlsx'
                logger.info('Saving report to ' + ofile)
                writer.dump(ofile)
                name = config[section]['contact_name']
                email_addr = config[section]['contact_email']
                firstname = name.split()[0]
                from_addr = formataddr((smtp_from, smtp_user))
                to_addr = formataddr((name, email_addr))
                # to_addr = formataddr((name, override_mail))
                if warn_students:
                    markup_txt = ptw.HtmlTableWriter()
                    markup_txt.headers = headers
                    markup_txt.value_matrix = warn_students
                    warn_text = markup_txt.dumps()
                else:
                    warn_text = ''
                msg_dict = {'firstname': firstname, 'course': course,
                            'name_from': smtp_from, 'warn_text': warn_text}
                subject = 'Weekly report for {course} course'.format(course=course)
                message = render_template(os.path.join(templates_dir, 'weekly_report.html'),
                                          msg_dict)
                since_last_notif = (today - api.get_last_notif(email_addr)).days
                logger.info('Preparing the message for ' + to_addr)
                mail = Email(from_=from_addr, to=to_addr, cc=cc_addr,
                             message_type='html', subject=subject, message=message,
                             attachments=[ofile], message_encoding="utf-8")
                if args.dryrun is not True:
                    if since_last_notif >= notif_freq:
                        logger.info('Sending report to {}'.format(to_addr))
                        server.send(mail, bcc=smtp_user)
                        api.set_last_notif(email_addr, today)
                else:
                    logger.info('[DRYRUN] Not sending email to {}'.format(to_addr))

    server.close()


def main():
    args = parse_arguments()
    remind_app(args)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# coding: utf-8

# This little project is hosted at: <https://gist.github.com/1455741>
# Copyright 2011-2020 Álvaro Justen [alvarojusten at gmail dot com]
# License: GPL <http://www.gnu.org/copyleft/gpl.html>

import os
import sys
from email.encoders import encode_base64
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from mimetypes import guess_type
from smtplib import SMTP


def render_template(template, vardict):
    """ renders a Jinja template into HTML """
    # check if template exists
    if not os.path.exists(template):
        print('No template file present: %s' % template)
        sys.exit()

    import jinja2
    template_loader = jinja2.FileSystemLoader(searchpath='/')
    template_env = jinja2.Environment(loader=template_loader)
    templ = template_env.get_template(template)
    return templ.render(vardict)


def get_email(email):
    if email is not None:
        if '<' in email:
            data = email.split('<')
            email = data[1].split('>')[0].strip()
            email.strip()
    else:
        email = ''
    return email



class Email(object):
    def __init__(self, from_, to, subject, message, message_type='plain',
                 attachments=None, cc=None, bcc=None, message_encoding='us-ascii'):
        self.email = MIMEMultipart()
        self.email['From'] = from_
        self.email['To'] = to
        self.email['Subject'] = subject
        self.bcc = bcc
        if cc is not None:
            self.email['Cc'] = cc
        text = MIMEText(message, message_type, message_encoding)
        self.email.attach(text)
        if attachments is not None:
            for filename in attachments:
                mimetype, encoding = guess_type(filename)
                mimetype = mimetype.split('/', 1)
                fp = open(filename, 'rb')
                attachment = MIMEBase(mimetype[0], mimetype[1])
                attachment.set_payload(fp.read())
                fp.close()
                encode_base64(attachment)
                attachment.add_header('Content-Disposition', 'attachment',
                                      filename=os.path.basename(filename))
                self.email.attach(attachment)

    def __str__(self):
        return self.email.as_string()


class EmailConnection(object):
    def __init__(self, server, username, password):
        if ':' in server:
            data = server.split(':')
            self.server = data[0]
            self.port = int(data[1])
        else:
            self.server = server
            self.port = 25
        self.username = username
        self.password = password
        self.connection = SMTP(self.server, self.port)
        self.connect()

    def connect(self):
        self.connection.ehlo()
        self.connection.starttls()
        self.connection.ehlo()
        self.connection.login(self.username, self.password)

    def send(self, message, from_=None, to=None, bcc=None):
        if type(message) == str:
            if from_ is None or to is None:
                raise ValueError('You need to specify `from_` and `to`')
            else:
                from_ = get_email(from_)
                to = get_email(to)
        else:
            from_ = message.email['From']
            if 'Cc' not in message.email:
                message.email['Cc'] = ''
            to_emails = [message.email['To']] + message.email['Cc'].split(',') + [bcc]
            to = [get_email(complete_email) for complete_email in to_emails]
            message = str(message)
        return self.connection.sendmail(from_, to, message)

    def close(self):
        self.connection.close()



from .course import Course


class School:
    def __init__(self, api):
        """
        A class that represents the school connected with this account

        :param api: TeachableAPI
        """
        self.api = api
        self._courses = None
        self._info = None
        self._id = None
        self._name = None
        self._users = None

    def get_course_list(self):
        raw_course_list = self.api.get_course_list()
        course_list = []
        for courseData in raw_course_list:
            course_list.append(Course(self.api, courseData.get('id')))
        return course_list

    @property
    def users(self):
        if not self._users:
            self._users = self.api.get_all_users()
        return self._users

    @property
    def id(self):
        if not self._id:
            self._id = self.info['id']
        return self._id

    @property
    def name(self):
        if not self._name:
            self._name = self.info['name']
        return self._name

    @property
    def info(self):
        if not self._info:
            self._info = self.api.get_school_info()
        return self._info

    @property
    def courses(self):
        if not self._courses:
            raw_course_list = self.api.get_course_list()
            self._courses = []
            for courseData in raw_course_list:
                self._courses.append(Course(self.api, courseData.get('id')))
        return self._courses

    def get_course_with_id(self, course_id):
        val = None
        for course in self.get_course_list():
            if course.id == course_id:
                val = course
        return val

    def __str__(self):
        return '{} (id:{})'.format(self.name, self.id)

    def __repr__(self):
        return '<School({})>'.format(self.id)


# coding: utf8
import argparse
import logging
import logging.config

import pyexcel as px

from ..api import Teachable
from ..course import Course
from ..user import User


def parse_arguments():
    parser = argparse.ArgumentParser(description='''Mass enroll users from Excel or CSV file into a specified course''',
                                     epilog="""---""")
    parser.add_argument('input_file', nargs=1,
                        help='Excel or CSV file. The only needed columns are \'fullname\' and \'email\' ')
    parser.add_argument('courseId', type=str, nargs=1, help='The id of the course they should be enrolled in')
    args = parser.parse_args()
    return args


def enroll_app(args):
    api = Teachable()
    logger = logging.getLogger(__name__)
    course_id = args.courseId[0]
    input_file = args.input_file[0]
    records = px.get_records(file_name=input_file)

    for user in records:
        # search if the user with the given email exists
        if user['email'] != '':
            u = User(api, user['email'])
            course = Course(api, course_id)
            if u.exists:
                resp = u.enroll(course)
                if 'message' in resp.keys():
                    logger.info(resp['message'])
                else:
                    logger.info(u.name + ' signed up!')
            else:
                logger.info('User {} doesn\'t exist. Creating and registering'.format(user['fullname']))
                # Add the user to the school and register to the course otherwise
                u.name = user['fullname']
                resp = u.create(course)
                if 'message' in resp.keys():
                    logger.info(resp['message'])


def main():
    args = parse_arguments()
    enroll_app(args)


if __name__ == '__main__':
    main()



import datetime

import requests
import json
import sys
import shelve
import os.path
import os
import time
import logging.config
from configparser import ConfigParser
import logging
from .school import School
from .user import User
from .course import Course
import re
# from appdirs import *


class Teachable:
    URL_COURSES = '/api/v1/courses'
    URL_GET_ALL_USERS = '/api/v1/users'
    URL_SCHOOL_INFO = '/api/v1/school'
    URL_FIND_USER = '/api/v1/users?name_or_email_cont='
    URL_COURSE_INFO = '/api/v1/courses/COURSE_ID'
    URL_REPORT_CARD = '/api/v1/users/USER_ID/report_card'
    URL_COURSE_REPORT = '/api/v1/users/USER_ID/course_report'
    URL_COURSE_USERS = '/api/v1/users?enrolled_in_specific%5B%5D=COURSE_ID'
    URL_COURSE_COMPLETED = '/api/v1/users?completed_course_in_any[]=COURSE_ID'
    URL_CURRICULUM = '/api/v1/courses/COURSE_ID/curriculum'
    URL_COURSE_LECTURES = '/api/v1/courses/COURSE_ID/lectures'
    URL_FIND_COURSE = '/api/v1/courses?name_cont='
    URL_COURSE_PRODUCTS = '/api/v1/courses/COURSE_ID/products'
    URL_IMPORT_USERS = '/api/v1/import/users'
    URL_ENROLLMENTS_USER = '/api/v1/enrollments?is_active=true&user_id=USER_ID'
    URL_LEADERBOARD = '/api/v1/courses/COURSE_ID/leaderboard.csv?page=1&per=PER_PAGE'
    URL_COURSE_PROGRESS = '/api/v1/course_progresses?user_id=USER_ID'  # no idea what does it do
    URL_PAGES_CERTIFICATE = '/api/v1/pages?feature=certificate'  # no idea what does it do
    URL_ENROLL_USER = '/api/v1/users/USER_ID/enrollments'
    # URL_ENROLL_USER = '/api/v1/users/USER_ID/enrollments/COURSE_ID'  #
    #    -data-binary '{"course_id":int,"user_id":int}' \
    URL_ENROLLED_USER = '/admin/users/USER_ID/enrolled'
    URL_UNENROLL_USER = '/api/v1/enrollments/unenroll'
    URL_STATEMENTS = '/api/v1/earnings_statements?sort_key=period_start&sort_direction=desc'

    TEACHABLE_ETC_DIR = os.path.join(sys.prefix, 'teachable', 'etc')
    TEACHABLE_CACHE_DIR = os.path.join(sys.prefix, 'teachable', 'cache')
    TEACHABLE_LOG_DIR = os.path.join(sys.prefix, 'teachable', 'log')
    TEACHABLE_TEMPLATES_DIR = os.path.join(sys.prefix, 'teachable', 'templates')
    DEFAULT_DIRS = {'TEACHABLE_LOG_DIR': TEACHABLE_LOG_DIR,
                    'TEACHABLE_ETC_DIR': TEACHABLE_ETC_DIR,
                    'TEACHABLE_CACHE_DIR': TEACHABLE_CACHE_DIR,
                    'TEACHABLE_TEMPLATES_DIR': TEACHABLE_TEMPLATES_DIR}

    def __init__(self, config_file=None):
        # Initializing all the needed directories
        for defaultdir in self.DEFAULT_DIRS.keys():
            logging.basicConfig(level=logging.DEBUG)
            tmp_log = logging.getLogger(__name__)
            os.environ[defaultdir] = self.DEFAULT_DIRS[defaultdir]
            if not os.path.exists(self.DEFAULT_DIRS[defaultdir]):
                tmp_log.debug('{} not found -> Creating'.format(self.DEFAULT_DIRS[defaultdir]))
                os.makedirs(self.DEFAULT_DIRS[defaultdir])
        self.logger = self.setup_logging(os.path.join(self.DEFAULT_DIRS['TEACHABLE_ETC_DIR'], 'logconf.ini'))
        self.usecache = True
        self._school = None
        self._courses = None
        self.site_url = None
        self.session = None
        self.cached_data = None
        if not config_file:
            conf_file = os.path.join(self.DEFAULT_DIRS['TEACHABLE_ETC_DIR'], 'config.ini')
        else:
            conf_file = config_file
        self.config = self.get_config(conf_file)
        if self.config:
            defaults = self.config['DEFAULT']
            cache_dir = self.DEFAULT_DIRS['TEACHABLE_CACHE_DIR']
            if not os.path.exists(cache_dir):
                self.logger.debug('{} not found -> Creating'.format(cache_dir))
                os.makedirs(cache_dir)
            notif_path = os.path.join(cache_dir, defaults['notifications_db'])
            statement_path = os.path.join(cache_dir, defaults['statements_db'])
            self.notif_status = shelve.open(notif_path)
            self.statement_status = shelve.open(statement_path)
            self.logger.debug('Using {} as notifications_db'.format(notif_path))
            self.logger.debug('Using {} as statements_db'.format(statement_path))
            self.site_url = defaults['site_url']
            self.email_regex = re.compile(defaults['email_regex'])
            self.session = requests.Session()
            self.session.auth = (defaults['username'], defaults['password'])
            self.cache_expire = defaults['cache_expire']
            self.cache_file = defaults['cache_file']
            # self.session.headers.update({'x-test': 'true'})
            self.session.headers.update({'Origin': self.site_url})
            self.expire_cache(os.path.join(cache_dir, self.cache_file))
            self.prepare_cache(os.path.join(cache_dir, self.cache_file))
        else:
            sys.exit(1)

    def __del__(self):
        if self.config:
            if self.cached_data:
                self.cached_data.sync()
                self.cached_data.close()
            if self.notif_status:
                self.notif_status.sync()
                self.notif_status.close()
            if self.statement_status:
                self.statement_status.sync()
                self.statement_status.close()

    @staticmethod
    def setup_logging(logconf):
        if os.path.exists(logconf):
            logging.debug('Found logconf in {}'.format(logconf))
            logging.config.fileConfig(fname=logconf, disable_existing_loggers=False)
        else:
            logging.error('Log conf doesn\'t exist [{}]'.format(logconf))
            logging.error('we are in dir {}, sys.prefix={}'.format(os.getcwd(), sys.prefix))
            logging.info('Using default logging configuration')
        return logging.getLogger(__name__)

    def get_config(self, configfile):
        """Gets config options"""
        if os.path.exists(configfile):
            self.logger.debug('Found config file at {}'.format(configfile))
            config = ConfigParser()
            config.read(configfile)
        else:
            self.logger.error('Missing config.ini file with login data (tried to find {})'.format(configfile))
            config = None
        return config

    def check_email(self, email):
        check = self.email_regex.fullmatch(email)
        if not check:
            self.logger.error('{} is not a valid email'.format(email))
        return check

    def prepare_cache(self, cache_path):
        self.logger.debug('Using cache {}'.format(cache_path))
        self.cached_data = shelve.open(cache_path)

    def expire_cache(self, cache_path):
        if os.path.isfile(cache_path):
            cache_antiquity = time.time() - os.path.getctime(cache_path)
            maximum_cache_duration = 60 * 60 * 24 * int(self.cache_expire)  # One day (rate limits
            #                                            are not that
            #                                            aggressive I hope)
            if cache_antiquity > maximum_cache_duration:
                os.remove(cache_path)
                self.logger.warning('Cache file dumped!')

    @property
    def school(self):
        if not self._school:
            self._school = School(self)
        return self._school

    @property
    def courses(self):
        if not self._courses:
            self._courses = self.school.courses
        return self._courses

    def get_leaderboard_csv(self, course, filename):
        """Fetches the leaderboard CSV list directly from Teachable and saves it as a CSV file

        :param course: Course object
        :param filename: name of the CSV file to use as output
        """
        per_page = '100000'  # includes in the leaderboard CSV as many as PER_PAGE users
        path = self.URL_LEADERBOARD.replace('COURSE_ID', str(course.id)).replace('PER_PAGE', str(per_page))
        full_url = self.site_url + path
        r = self.session.get(full_url, allow_redirects=True)
        if filename == '':
            filename = 'leaderboard_{course}.csv'.format(course=course.name)
        open(filename, 'wb').write(r.content)

    def get_user_course_report(self, user_id):
        path = self.URL_COURSE_REPORT.replace('USER_ID', str(user_id))
        return self._get_json_at(path).get('report')

    def get_course_sections(self, course_id):
        url_course_curriculum = self.URL_CURRICULUM.replace('COURSE_ID', str(course_id))
        return self._get_json_at(url_course_curriculum).get('lecture_sections')

    def get_course_list(self):
        course_info = self._get_json_at(self.URL_COURSES)
        return course_info.get('courses')

    def get_earning_statements(self):
        statement = self._get_json_at(self.URL_STATEMENTS)
        return statement.get('earnings_statements')

    def get_course_info(self, course_id):
        url_course_info = self.URL_COURSE_INFO.replace('COURSE_ID', str(course_id))
        course_info = self._get_json_at(url_course_info)
        return course_info

    def get_school_info(self):
        school_info = self._get_json_at(self.URL_SCHOOL_INFO)
        return school_info

    def find_user(self, email: str) -> User or None:
        """Searches for a specific user, the API uses the same endpoint, for
        one or many

        :param email: email address of the user you are searching
        :return: the user you are searching
        """
        user_list = self._get_json_at(self.URL_FIND_USER + email).get('users')
        if len(user_list) == 0:
            return None
        else:
            return User(self, user_list[0]['email'])

    def get_user_info(self, email: str) -> json:
        """Searches for a specific user, the API uses the same endpoint, for
        one or many

        :param email: email address of the user you are searching
        :return: the json with the info of the user you are searching
        """
        user_list = self._get_json_at(self.URL_FIND_USER + email).get('users')
        if len(user_list) == 0:
            return []
        else:
            return user_list[0]

    def find_many_users(self, email: str) -> list[User]:
        """Searches for multiple users, the API uses the same endpoint for one
        or many

        :param email: part of email address of the user you are searching (e.g. @gmail.com)
        :return: list of users you are searching
        """
        user_list = self._get_json_at(self.URL_FIND_USER + email).get('users')
        if len(user_list) == 0:
            return []
        else:
            return [User(self, user['email']) for user in user_list]

    def get_all_users(self) -> list[User]:
        """Gets all the Users registered to the school

        :return: list of all users in the school
        :rtype: [User]
        """
        user_list = self._get_json_at(self.URL_GET_ALL_USERS).get('users')
        if len(user_list) == 0:
            return []
        else:
            return [User(self, user['email']) for user in user_list]

    def get_last_notif(self, email: str) -> datetime.date:
        """Returns the date of the last notification sent to the user

        :param email: email of the user you are interested into
        :return: last notification date
        :rtype: datetime.date
        """
        try:
            notified = self.notif_status[email]
            self.logger.debug('{} was sent a notification last time on {}'.format(email, notified))
        except KeyError:
            self.logger.debug('{} was never sent a notification'.format(email))
            notified = datetime.date(1970, 1, 1)
        return notified

    def set_last_notif(self, email, newdate):
        self.notif_status[email] = newdate
        self.notif_status.sync()
        
    def get_last_statement_date(self) -> datetime.date:
        """Returns the date of the last statement retrieved

        :return: last statement date
        :rtype: datetime.date
        """
        try:
            last_statement = self.statement_status['last']
            self.logger.debug('Last statement was dated {}'.format(last_statement))
        except KeyError:
            self.logger.debug('No statement was ever retrieved')
            last_statement = datetime.date(1970, 1, 1)
        return last_statement

    def set_last_statement_date(self, newdate: datetime):
        self.statement_status['last'] = newdate
        self.statement_status.sync()
        
    def find_courses(self, course):
        """Searches for courses containing the specific text

        :param course: Course
        :return: Course list
        :rtype: list(Course)
        """
        course_list = self._get_json_at(self.URL_FIND_COURSE +
                                        course).get('courses')
        if len(course_list) == 0:
            return None
        else:
            courses = []
            for course_data in course_list:
                courses.append(Course(self, course_data.get('id')))
            return courses

    def get_course_price(self, course_id):
        products = self.get_course_products(course_id)
        if len(products) > 0:
            return products[0].get('price')
        else:
            return 0

    def get_course_products(self, course_id):
        path = self.URL_COURSE_PRODUCTS.replace('COURSE_ID', str(course_id))
        result = self._get_json_at(path)
        return result.get('products')

    def get_course_users(self, course_id):
        path = self.URL_COURSE_USERS.replace('COURSE_ID', str(course_id))
        users = self._get_json_at(path).get('users')
        if len(users) == 0:
            return None
        else:
            return [User(self, user['email']) for user in users]

    def get_course_users_completed(self, course_id):
        path = self.URL_COURSE_COMPLETED.replace('COURSE_ID', str(course_id))
        users = self._get_json_at(path).get('users')
        if len(users) == 0:
            return None
        else:
            return [User(self, user['email']) for user in users]

    def get_user_report_card(self, user_id):
        """Gets the full report card fot userId, returning the full list of
        lessons the user has completed"""
        path = self.URL_REPORT_CARD.replace('USER_ID', str(user_id))
        return self._get_json_at(path)

    def add_users_to_school(self, users_array, course_id):
        users_json_array = []
        for userRow in users_array:
            if self.check_email(userRow['email']):
                user_json = {
                    "email": userRow['email'],
                    "name": userRow['fullname'],
                    "password": None,
                    "role": "student",
                    "course_id": course_id,
                    "unsubscribe_from_marketing_emails": 'false'
                }
                users_json_array.append(user_json)
        payload = {
            "user_list": users_json_array,
            "course_id": course_id,
            "coupon_code": None,
            "users_role": "student",
            "author_bio_data": {}
        }
        resp = self._post_json_at(self.URL_IMPORT_USERS, json.dumps(payload))
        return json.loads(resp)

    def add_user_to_school(self, user: User, course: Course = None) -> str:
        """
        Adds a User to a school and enroll to a specific Course if provided

        :param user: User object
        :param course: Course object (optional)
        :return: json
        """
        users_json_array = []
        if self.check_email(user.email):
            user_json = {
                "email": user.email,
                "name": user.name,
                "password": None,
                "role": "student",
                "unsubscribe_from_marketing_emails": 'false'
            }
            if course:
                user_json["course_id"] = course.id
            users_json_array.append(user_json)
        payload = {
            "user_list": users_json_array,
            "coupon_code": None,
            "users_role": "student",
            "author_bio_data": {}
        }
        if course:
            payload["course_id"] = course.id
        resp = self._post_json_at(self.URL_IMPORT_USERS, json.dumps(payload))
        return json.loads(resp)

    def enroll_user_to_course(self, user: User, course: Course) -> str:
        path = self.URL_ENROLL_USER.replace('USER_ID', str(user.id))
        json_body = json.dumps({"course_id": int(course.id)})
        response = self._post_json_at(path, json_body)
        # Now refreshing the status in the cache
        self.usecache = False
        self.get_enrolled_courses(user.id)
        self.usecache = True
        if response:
            return json.loads(response)
        else:
            return response

    def unenroll_user_from_course(self, user: User, course: Course) -> str:
        """
        Unenrolls a user from a course.

        :param user: User object
        :param course: Course object
        :return: response of the operation
        """
        path = self.URL_UNENROLL_USER
        json_body = json.dumps({"course_id": int(course.id), "user_id": int(user.id)})
        response = self._put_json_at(path, json_body)
        # updating also the cache for the enrolled courses
        self.usecache = False
        self.get_enrolled_courses(user.id)
        self.usecache = True
        if response:
            return json.loads(response)
        else:
            return response

    def get_enrolled_courses(self, user_id: str) -> list[Course]:
        """Gets the courses the user is enrolled in and returns it as a list

        :param user_id: user id
        :return: list of Course objects
        """
        path = self.URL_ENROLLMENTS_USER.replace('USER_ID', str(user_id))
        courses = self._get_json_at(path).get('enrollments')
        return [Course(self, c['course_id']) for c in courses]

    def check_enrollment_to_course(self, user_id: str, course_id: str) -> bool:
        """Check if a user is enrolled in a specific course

        :param user_id: User id
        :param course_id: Course id
        :return: True if enrolled, False if not enrolled
        """
        courses = self.get_enrolled_courses(user_id)
        return int(course_id) in [p.id for p in courses]

    def _get_json_at(self, path):
        if self.usecache and path in self.cached_data:
            self.logger.debug(("Found cached data for " + path))
            return self.cached_data[path]
        else:
            full_url = self.site_url + path
            self.logger.debug(("Downloading data from " + full_url))
            json_data = self.session.get(full_url).json()
            if json_data.get('error'):
                self.logger.error('Check Teachable credentials')
                sys.exit(1)
            self.logger.debug('Updating cache data for ' + path)
            self.cached_data[path] = json_data
            return json_data

    def _post_json_at(self, path, json_body):
        full_url = self.site_url + path
        self.logger.debug(("Uploading POST data to " + full_url))
        self.logger.debug("JSON Body : " + json_body)
        json_txt = json.loads(json_body)
        self.session.headers.update({'Content-Type': 'application/json;charset=UTF-8'})
        self.logger.debug((json.dumps(json_txt, sort_keys=True, indent=4, separators=(',', ': '))))
        json_response_body = self.session.post(full_url, data=json_body)
        return json_response_body.text

    def _put_json_at(self, path, json_body):
        full_url = self.site_url + path
        self.logger.debug(("Uploading PUT data to " + full_url))
        self.logger.debug("JSON Body : " + json_body)
        json_txt = json.loads(json_body)
        self.session.headers.update({'Content-Type': 'application/json;charset=UTF-8'})
        self.logger.debug((json.dumps(json_txt, sort_keys=True, indent=4, separators=(',', ': '))))
        json_response_body = self.session.put(full_url, data=json_body)
        return json_response_body.text



import datetime


class Course:
    def __init__(self, teachable_api, course_id):
        self.api = teachable_api
        self.id = course_id
        self._name = None
        self._price = None
        self._sections = None
        self._lectures = None
        self._users = None
        self._completed = None

    @property
    def price(self):
        if not self._price:
            self._price = self.api.get_course_price(self.id)
        return self._price

    @property
    def users(self):
        if not self._users:
            self._users = self.api.get_course_users(self.id)
        return self._users

    @property
    def completed(self):
        if not self._completed:
            self._completed = self.api.get_course_users_completed(self.id)
        return self._completed

    @property
    def name(self):
        if not self._name:
            info = self.api.get_course_info(self.id)
            self._name = info.get('name')
        return self._name

    def get_sections(self):
        sections = self.api.get_course_sections(self.id)
        sectionslist = []
        for sectionJson in sections:
            sectionslist.append(Section(sectionJson))
        return sectionslist

    @property
    def sections(self):
        if not self._sections:
            sections = self.api.get_course_sections(self.id)
            self._sections = []
            for sectionJson in sections:
                self._sections.append(Section(sectionJson))
        return self._sections

    def get_lectures(self):
        """Returns the list of all the lectures as CourseSectionLecture objects"""
        lectures = []
        if self.sections:
            for section in self.sections:
                lectures.extend(section.lectures)
        return lectures

    @property
    def lectures(self):
        if not self._lectures:
            self._lectures = []
            if self.sections:
                for section in self.sections:
                    self._lectures.extend(section.lectures)
        return self._lectures

    def get_lecture_with_id(self, lecture_id):
        for section in self.get_sections():
            lecture = section.get_lecture_with_id(lecture_id)
            if lecture:
                return lecture
        return None

    def __str__(self):
        return '{} (id:{})'.format(self.name, self.id)

    def __repr__(self):
        return '<Course({})>'.format(self.id)


class Section:
    def __init__(self, json_data):
        self.id = json_data.get('id')
        self.name = json_data.get('name')
        self.lectures = []
        lectures_json = json_data.get('lectures')
        for rawLecture in lectures_json:
            self.lectures.append(Lecture(rawLecture))

    def get_lecture_with_id(self, lecture_id):
        for lecture in self.lectures:
            if lecture.id == lecture_id:
                return lecture
        return None

    def __str__(self):
        return '{} (id:{})'.format(self.name, self.id)

    def __repr__(self):
        return '<Section({})>'.format(self.id)


class Lecture:
    def __init__(self, json_data):
        self.id = json_data.get('id')
        self.name = json_data.get('name')
        self.attachments = json_data.get('attachments')
        self.duration = 0
        for attachment in self.attachments:
            self.duration += attachment.get('duration')
        self.duration_as_text = str(datetime.timedelta(seconds=self.duration))

    def __str__(self):
        return '{} (id:{})'.format(self.name, self.id)

    def __repr__(self):
        return '<Lecture({})>'.format(self.id)



"""A setuptools based setup module.
See:
https://packaging.python.org/guides/distributing-packages-using-setuptools/
https://github.com/pypa/sampleproject
Modified by Madoshakalaka@Github (dependency links added)
"""

# io.open is needed for projects that support Python 2.7
# It ensures open() defaults to text mode with universal newlines,
# and accepts an argument to specify the text encoding
# Python 3 only projects can skip this import
from io import open
from os import path

# Always prefer setuptools over distutils
from setuptools import setup, find_namespace_packages

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="teachable-school-manager",  # Required
    version="1.5",  # Required
    description="Manage your Teachable school using the unofficial Teachable API",  # Optional
    long_description=long_description,  # Optional
    long_description_content_type="text/markdown",  # Optional (see note above)
    url="https://github.com/stezz/teachable-scripts",  # Optional
    author="Stefano Mosconi",  # Optional
    author_email="stefano.mosconi@gmail.com",  # Optional
    package_dir={"": "src"},
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        "Development Status :: 4 - Beta",
        # Indicate who your project is intended for
        "Intended Audience :: Developers",
        "Topic :: Education",
        # Pick your license as you wish
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
    keywords="scripts teachable",  # Optional
    packages=find_namespace_packages("src", exclude=["contrib", "docs", "tests"]),  # Required
    # packages=["teachable"],  # Required
    python_requires=">3.7, <4",
    install_requires=[
        "jinja2==2.11.2",
        "pyexcel==0.6.6",
        "pyexcel-xlsx==0.6.0",
        "pytablewriter[excel,html]==0.58.0",
        "requests==2.25.0",
        "schedule==0.6.0",
    ],  # Optional
    extras_require={"dev": []},  # Optional
    dependency_links=[],
    # these directories need to be synced with whatever is in TeachableAPI as DEFAULT_DIRS
    data_files=[(path.join("teachable", "etc" ), ["src/etc/config_example.ini", "src/etc/logconf.ini"]),
                (path.join('teachable', 'templates'), ["templates/email_inactive.txt",
                                                       "templates/email_notstarted.txt",
                                                       "templates/weekly_report.html"])],
    entry_points={"console_scripts": ["teachable_leaderboard=teachable.scripts.leaderboard:main",
                                      "teachable_user_report=teachable.scripts.user_report:main",
                                      "teachable_remind=teachable.scripts.remind:main",
                                      "teachable_enroll=teachable.scripts.enroll:main",
                                      "teachable_scheduler=teachable.scripts.scheduler:main",
                                      "teachable_statements=teachable.scripts.statements:main"]},  # Optional
    # scripts=["scripts/remind.py"],  # Optional
    project_urls={  # Optional
        "Bug Reports": "https://github.com/stezz/teachable-scripts/issues",
        #        "Funding": "https://donate.pypi.org",
        #        "Say Thanks!": "http://saythanks.io/to/example",
        "Source": "https://github.com/stezz/teachable-scripts/",
    },
)



# coding: utf8
import argparse
import datetime
import logging
import logging.config
import sys
from email.utils import formataddr

from ..utils.email_utils import Email
from ..utils.email_utils import EmailConnection

import pytablewriter as ptw

from ..api import Teachable


def parse_arguments():
    parser = argparse.ArgumentParser(description='''Get the latest earning statement as Excel (default) or CSV. 
                            Optionally send them over email to the school owner''')
    parser.add_argument('--email', '-e',
                        action='store_true',
                        default='False', help='''[Optional] Send the statements to the school owner''')
    parser.add_argument('--all', '-a',
                        action='store_true',
                        default='False', help='''[Optional] Get all the statements, not just the last one''')
    parser.add_argument('--format', '-f', nargs='?', default='excel',
                        help='Output format (excel[default], csv)')
    parser.add_argument('--ofile', '-o', nargs='?', default='earnings_statement', help='[Optional] Output file name')

    args = parser.parse_args()
    return args


def statements_app(args):
    api = Teachable()
    logger = logging.getLogger(__name__)
    send_email = args.email
    get_all = args.all
    data = []
    headers = []
    last_statement = api.get_last_statement_date()
    statements = api.get_earning_statements()

    if statements:
        keys = statements[0].keys()
        # Dropping the 'meta' key since we don't need it
        keys = [x for x in keys if not x == 'meta']
    else:
        logger.error('No earning statements available for your school')
        sys.exit(0)

    if get_all is not True:
        # By default we only get the latest statement, which is first in the list
        statements = statements[0:1]

    for row in statements:
        # search if the user with the given email exists
        row_data = []
        for k in keys:
            if k == 'internal_gateway_amount' or \
                    k == 'teachable_payments_amount' or \
                    k == 'custom_gateway_amount' or \
                    k == 'total_amount':
                # For whatever reason this is an int with USD value*100, rather than a str :/
                row_data.append(row[k] / 100)
            else:
                row_data.append(row[k])

        updated = datetime.datetime.strptime(row['updated_at'], '%Y-%m-%dT%H:%M:%SZ').date()
        # When we want all statements we really want all, disregarding the statements_db
        if updated > last_statement or get_all is True:
            logger.debug('New statement found, updated on {}'.format(updated))
            data.append(row_data)
        else:
            logger.debug('This statement was already retrieved, skipping (last statement date: {}'.
                         format(last_statement))

    # Saving the last statement only if we are running in default mode
    if get_all is not True:
        newest = datetime.datetime.strptime(statements[0]['updated_at'], '%Y-%m-%dT%H:%M:%SZ').date()
        api.set_last_statement_date(newest)

    if data:
        for k in keys:
            if k == 'internal_gateway_amount' or \
                    k == 'teachable_payments_amount' or \
                    k == 'custom_gateway_amount' or \
                    k == 'total_amount':
                # Making it explicit that we are talking about amounts in USD
                headers.append(k + '_usd')
            else:
                headers.append(k)

        if args.format == 'csv':
            writer = ptw.CsvTableWriter()
        else:
            writer = ptw.ExcelXlsxTableWriter()
            # Because honestly MS Gothic sucks as a font
            writer.format_table['cell']['font_name'] = 'Calibri'
            writer.format_table['cell']['font_size'] = 12
            writer.format_table['header']['font_name'] = 'Calibri'
            writer.format_table['header']['font_size'] = 12

        writer.table_name = 'Earning Statements Teachable School'
        writer.headers = headers
        writer.value_matrix = data

        if args.format == 'csv':
            ofile = args.ofile + '.csv'
            logger.info('Saving to {}'.format(ofile))
            with open(ofile, 'w') as f:
                f.write(writer.dumps())
            f.close()
        else:
            ofile = args.ofile + '.xlsx'
            logger.info('Saving to {}'.format(ofile))
            writer.dump(ofile)

        if send_email is True:
            config = api.config
            defaults = config['DEFAULT']
            smtp_pwd = defaults['smtp_pwd']
            smtp_user = defaults['smtp_user']
            smtp_port = defaults['smtp_port']
            smtp_server = defaults['smtp_server']
            smtp_from = defaults['smtp_from']

            logger.debug('Connecting to email server ({})'.format(smtp_server))
            server_str = smtp_server + ':' + str(smtp_port)
            server = EmailConnection(server_str, smtp_user, smtp_pwd)

            to_addr = formataddr((smtp_from, smtp_user))
            from_addr = formataddr((smtp_from, smtp_user))
            subject = 'Teachable earning statements'
            message = 'Here the earning statements report.'
            logger.debug('Preparing the message for ' + to_addr)
            mail = Email(from_=from_addr, to=to_addr,
                         message_type='html', subject=subject, message=message,
                         attachments=[ofile], message_encoding="utf-8")
            logger.info('Sending earning statements to {}'.format(to_addr))
            server.send(mail)
            server.close()
    else:
        logger.info('No new statements were found')


def main():
    args = parse_arguments()
    statements_app(args)


if __name__ == '__main__':
    main()



# coding: utf8
import argparse
import logging
from ..user import User
from ..api import Teachable
import pytablewriter as ptw


def parse_arguments():
    parser = argparse.ArgumentParser(description='''Get your Teachable students
    report. 
    
    By default it will generate a progress summary report of all the students that
    are enrolled in all your courses. 
    
    This is very similar to using leaderboard.py: while this allows to
    specify a specific set of users, the other one allows to specify a course.
    
    Pay attention if you have a lot of students
    because this will be rate limited at some point''')
    group = parser.add_mutually_exclusive_group()
    group.add_argument('--emails', '-e', type=str, nargs='+', default='',
                       help='list of emails (separated by spaces) - cannot be used with -s')
    parser.add_argument('--output_file', '-o', nargs='?', default='teachable_stats', help='Output file')
    group.add_argument('--search', '-s', nargs='?', default='',
                       help='''Searches specific text in name or email. For instance -s @gmail.com or
    -s *@gmail.com will look for all the users that have an email ending in
    @gmail.com. Or -s Jack will look for all the users that have Jack in
    their name (or surname) - cannot be used with -e''')
    parser.add_argument('--format', '-f', nargs='?', default='txt', help='Output format (excel, csv, txt [default])')
    parser.add_argument('--courseid', '-c', nargs='?', default=0, help='''Limit
    search to this course id only (numeric value)''')
    parser.add_argument('--detailed', '-d', action='store_true', default='False',
                        help='Get detailed progress report')

    # parser.add_argument('--hidefree', type=int, default=0, help='0: show/1: hide free courses ')

    args = parser.parse_args()
    return args


def report_app(args):
    # logger.info args

    # HIDE_FREE_COURSES = args.hidefree  # set to 0 to show all

    api = Teachable()
    logger = logging.getLogger(__name__)
    users_mails = []
    if args.emails:
        for email in args.emails:
            users_mails.append(email)

    if args.search:
        users_mails = [x.email for x in api.find_many_users(args.search)]

    if not users_mails:
        users_mails = [x.email for x in api.school.users]

    data = []
    for user_mail in users_mails:
        user = User(api, user_mail)
        if args.detailed is True:
            data += user.get_detailed_stats()
        else:
            data += user.get_summary_stats(int(args.courseid))

    if args.detailed is True:
        headers = ['User', 'Email', 'Date', 'Course', 'Chapter', 'Duration']
    else:
        headers = ['User', 'Email', 'Course', 'Updated at', 'Completed (%)', 'Days since last lesson']

    if args.format == 'csv':
        writer = ptw.CsvTableWriter()
    elif args.format == 'excel':
        writer = ptw.ExcelXlsxTableWriter()
        # Because honestly MS Gothic sucks as a font
        writer.format_table['cell']['font_name'] = 'Calibri'
        writer.format_table['cell']['font_size'] = 12
        writer.format_table['header']['font_name'] = 'Calibri'
        writer.format_table['header']['font_size'] = 12
    else:
        writer = ptw.MarkdownTableWriter()

    writer.table_name = 'Teachable Stats'
    writer.headers = headers
    writer.value_matrix = data

    if args.format == 'csv':
        ofile = args.output_file + '.csv'
        logger.info('Saving to ' + ofile)
        with open(ofile, 'w') as f:
            f.write(writer.dumps())
        f.close()
    elif args.format == 'excel':
        ofile = args.output_file + '.xlsx'
        logger.info('Saving to ' + ofile)
        writer.dump(ofile)
    else:
        writer.write_table()


def main():
    args = parse_arguments()
    report_app(args)


if __name__ == '__main__':
    main()
